/*
 *  arch/mips/emma3tl/common/irq.c
 *      This file is common irq dispatcher.
 *
 * Author: NEC Electronics Corporation
 *
 * Copyright (C) 2008 NEC Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  This file is based on the arch/mips/ddb5xxx/ddb5477/irq.c
 *
 *	Copyright 2001 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/types.h>

#include <asm/i8259.h>
#include <asm/system.h>
#include <asm/mipsregs.h>
#include <asm/debug.h>
#include <asm/addrspace.h>
#include <asm/bootinfo.h>

#include <asm/emma3tl/emma3tl.h>
#ifdef CONFIG_E21
#include <asm/emma3tl/e21.h>
#endif

/*
 * IRQ mapping
 *
 *  0-7: 8 CPU interrupts
 *	0 -	software interrupt 0
 *	1 - 	software interrupt 1
 *	2 - 	most Vrc5477 interrupts are routed to this pin
 *	3 - 	(optional) some other interrupts routed to this pin for debugg
 *	4 - 	not used
 *	5 - 	not used
 *	6 - 	not used
 *	7 - 	cpu timer (used by default)
 *
 */

extern void mips_cpu_irq_init(u32 base);

extern void emma_irq_init(u32 base);
extern void emma_sw_irq_init(u32 base);
extern asmlinkage void emma_handle_int(void);
extern asmlinkage void emma_irq_dispatch(struct pt_regs *regs);

static struct irqaction irq_cascade = {
	no_action, 0, CPU_MASK_NONE, "cascade", NULL, NULL
};

void __init arch_init_irq(void)
{

	db_run(printk(KERN_INFO "e21_irq_setup invoked.\n"));

	/* by default, interrupts are disabled. */
	emma_out32(EMMA_BHIF_INT0_EN_0, 0);
	emma_out32(EMMA_BHIF_INT0_EN_1, 0);
	emma_out32(EMMA_BHIF_INT0_EN_2, 0);
	emma_out32(EMMA_BHIF_INT1_EN_0, 0);
	emma_out32(EMMA_BHIF_INT1_EN_1, 0);
	emma_out32(EMMA_BHIF_INT1_EN_2, 0);
	emma_out32(EMMA_BHIF_SW_INT0_EN, 0);

	clear_c0_status(0xff00);
	set_c0_status(0x0400);

	/* init all controllers */
	emma_irq_init(EMMA_IRQ_BASE);
	emma_sw_irq_init(EMMA_SW_IRQ_BASE);
	mips_cpu_irq_init(CPU_IRQ_BASE);

	/* setup cascade interrupts */
	setup_irq(CPU_IRQ_BASE + CPU_EMMA_CASCADE, &irq_cascade);
	setup_irq(EMMA_SW_CASCADE, &irq_cascade);
}

asmlinkage void plat_irq_dispatch(struct pt_regs *regs)
{
	unsigned int pending = read_c0_status() & read_c0_cause();

	if (pending & STATUSF_IP7)
		do_IRQ(CPU_IRQ_BASE + 7, regs);
	else if (pending & STATUSF_IP2)
		emma_irq_dispatch(regs);
	else if (pending & STATUSF_IP1)
		do_IRQ(CPU_IRQ_BASE + 1, regs);
	else if (pending & STATUSF_IP0)
		do_IRQ(CPU_IRQ_BASE + 0, regs);
	else
		spurious_interrupt(regs);
}
