/*
 *  arch/mips/emma3tl/e21/platofrm.c
 *      This file sets up platform devices for EMMA3TL E21.
 *
 * Author: NEC Electronics Corporation
 *
 * Copyright (C) 2008 NEC Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  This file is based on the arch/mips/emma2rh/markeins/platofrm.c
 *
 *  Copyright(C) MontaVista Software Inc, 2006
 *
 *  Author: dmitry pervushin <dpervushin@ru.mvista.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/ioport.h>
#include <linux/serial_8250.h>
#include <linux/mtd/physmap.h>
#include <linux/mtd/partitions.h>

#include <linux/cpu.h>
#include <asm/bootinfo.h>
#include <asm/addrspace.h>
#include <linux/time.h>
#include <asm/bcache.h>
#include <linux/irq.h>
#include <linux/reboot.h>
#include <asm/gdb-stub.h>
#include <asm/traps.h>
#include <asm/debug.h>

#include <asm/emma3tl/emma3tl.h>
#include <linux/nec_candy_pd.h>
#include <linux/platform_device.h>

#if 1 // Add for EMMA2TL Pixela Board ES1 HASE@09061802 NOR Flash上にMTDパーティション作成
#define SZ_1K				1024
#define SZ_1M				SZ_1K * SZ_1K
// partition size for nor flash
#define KERNEL_SIZE_N		(3*SZ_1M + 512*SZ_1K)	// 3.5 Mbyte
#define ROOTFS_SIZE_N		(6*SZ_1M)				// 6.0 Mbyte
#define APP_BIN_SIZE_N		(2*SZ_1M + 512*SZ_1K)	// 2.5 Mbyte
#define B_LOADER_SIZE_N		(256*SZ_1K)				// 256 Kbyte
#define LOGOIMG_SIZE_N		(384*SZ_1K)				// 384 Kbyte was 512 Kbyte Mod by HASE@09103003
#define FW_SIZE_N			(3*SZ_1M)				// 3.0 Mbyte
#define B_SETTING_SIZE_N	(256*SZ_1K)				// 256 Kbyte was 128 Kbyte Mod by HASE@09103003
#define MAC_ADDR_SIZE_N		(128*SZ_1K)				// 128 Kbyte
#endif										// Total  16.0 Mbyte

#define I2C_EMMA "emma-iic" /* must be in sync with IIC driver */
#define SERIAL_EMMA "serial8250"

#define EMMA_SERIAL_CLOCK 24000000
#define EMMA_SERIAL_FLAGS (UPF_BOOT_AUTOCONF | UPF_SKIP_TEST)

static struct resource i2c_emma_resources_0[] = {
	{
		.name   = NULL,
		.start  = EMMA_IRQ_PIIC0,
		.end    = EMMA_IRQ_PIIC0,
		.flags  = IORESOURCE_IRQ
	}, {
		.name   = NULL,
		.start  = KSEG1ADDR(EMMA_PIIC0_BASE),
		.end    = KSEG1ADDR(EMMA_PIIC0_BASE + 0x1000),
		.flags  = 0
	}, {
		.name   = "pbrd",
		.start  = KSEG1ADDR(EMMA_PBRD_BASE),
		.end    = KSEG1ADDR(EMMA_PBRD_BASE + 0x1000),
		.flags  = 0
	},
};

static struct resource i2c_emma_resources_1[] = {
	{
		.name   = NULL,
		.start  = EMMA_IRQ_PIIC1,
		.end    = EMMA_IRQ_PIIC1,
		.flags  = IORESOURCE_IRQ
	}, {
		.name   = NULL,
		.start  = KSEG1ADDR(EMMA_PIIC1_BASE),
		.end    = KSEG1ADDR(EMMA_PIIC1_BASE + 0x1000),
		.flags  = 0
	}, {
		.name   = "pbrd",
		.start  = KSEG1ADDR(EMMA_PBRD_BASE),
		.end    = KSEG1ADDR(EMMA_PBRD_BASE + 0x1000),
		.flags  = 0
	},
};

static struct resource i2c_emma_resources_2[] = {
	{
		.name   = NULL,
		.start  = EMMA_IRQ_PIIC2,
		.end    = EMMA_IRQ_PIIC2,
		.flags  = IORESOURCE_IRQ
	}, {
		.name   = NULL,
		.start  = KSEG1ADDR(EMMA_PIIC2_BASE),
		.end    = KSEG1ADDR(EMMA_PIIC2_BASE + 0x1000),
		.flags  = 0
	}, {
		.name   = "pbrd",
		.start  = KSEG1ADDR(EMMA_PBRD_BASE),
		.end    = KSEG1ADDR(EMMA_PBRD_BASE + 0x1000),
		.flags  = 0
	},
};

struct platform_device i2c_emma_devices[] = {
	[0] = {
		.name = I2C_EMMA,
		.id = 0,
		.resource = i2c_emma_resources_0,
		.num_resources = ARRAY_SIZE(i2c_emma_resources_0),
	},
	[1] = {
		.name = I2C_EMMA,
		.id = 1,
		.resource = i2c_emma_resources_1,
		.num_resources = ARRAY_SIZE(i2c_emma_resources_1),
	},
	[2] = {
		.name = I2C_EMMA,
		.id = 2,
		.resource = i2c_emma_resources_2,
		.num_resources = ARRAY_SIZE(i2c_emma_resources_2),
	},
};

static struct  plat_serial8250_port platform_serial_ports[] = {
#if 1  /* PRD-LH132B基板では,  ttyS0(Subm通信用)=COM1  ttyS1(デバッグ用)=COM0。 暫定的に、ここで設定。*/
	[0] = {
		.membase = (void __iomem *)KSEG1ADDR(EMMA_PURT1_BASE + 3),
		.mapbase = KSEG1ADDR(EMMA_PURT1_BASE + 3),
		.irq = EMMA_IRQ_PUART1,
		.uartclk = EMMA_SERIAL_CLOCK,
		.private_data = (void *)(KSEG1ADDR(EMMA_PURT1_BASE + 3) + 0x7c),
		.regshift = 2,
		.iotype = UPIO_DWAPB,
		.flags = EMMA_SERIAL_FLAGS,
	},
	[1] = {
		.membase = (void __iomem *)KSEG1ADDR(EMMA_PURT0_BASE + 3),
		.mapbase = KSEG1ADDR(EMMA_PURT0_BASE + 3),
		.irq = EMMA_IRQ_PUART0,
		.uartclk = EMMA_SERIAL_CLOCK,
		.private_data = (void *)(KSEG1ADDR(EMMA_PURT0_BASE + 3) + 0x7c),
		.regshift = 2,
		.iotype = UPIO_DWAPB,
		.flags = EMMA_SERIAL_FLAGS,
	},
#else
	[0] = {
		.membase = (void __iomem *)KSEG1ADDR(EMMA_PURT0_BASE + 3),
		.mapbase = KSEG1ADDR(EMMA_PURT0_BASE + 3),
		.irq = EMMA_IRQ_PUART0,
		.uartclk = EMMA_SERIAL_CLOCK,
		.private_data = (void *)(KSEG1ADDR(EMMA_PURT0_BASE + 3) + 0x7c),
		.regshift = 2,
		.iotype = UPIO_DWAPB,
		.flags = EMMA_SERIAL_FLAGS,
	},
	[1] = {
		.membase = (void __iomem *)KSEG1ADDR(EMMA_PURT1_BASE + 3),
		.mapbase = KSEG1ADDR(EMMA_PURT1_BASE + 3),
		.irq = EMMA_IRQ_PUART1,
		.uartclk = EMMA_SERIAL_CLOCK,
		.private_data = (void *)(KSEG1ADDR(EMMA_PURT1_BASE + 3) + 0x7c),
		.regshift = 2,
		.iotype = UPIO_DWAPB,
		.flags = EMMA_SERIAL_FLAGS,
	},
#endif
	[2] = {
		.membase = (void __iomem *)KSEG1ADDR(EMMA_PURT2_BASE + 3),
		.mapbase = KSEG1ADDR(EMMA_PURT2_BASE + 3),
		.irq = EMMA_IRQ_PUART2,
		.uartclk = EMMA_SERIAL_CLOCK,
		.private_data = (void *)(KSEG1ADDR(EMMA_PURT2_BASE + 3) + 0x7c),
		.regshift = 2,
		.iotype = UPIO_DWAPB,
		.flags = EMMA_SERIAL_FLAGS,
	},
	[3] = {
		.flags = 0
	}
};

static struct  platform_device serial_emma = {
	.name = SERIAL_EMMA,
	.dev = {
		.platform_data = &platform_serial_ports,
	},
};

#if 1 // Mod for EMMA2TL Pixela Board ES1 HASE@09061802 MACアドレス格納位置を変更
#define EMMA3TL_MAC_FLASH	0xbffe0000 // /dev/mtd7 "OSBOURNE : MAC address" パーティション
#else
#define EMMA3TL_MAC_FLASH	0xbfe20000
#endif

static struct nec_candy_platform_data nec_candy_pdata = {
	.pmd_addr = 0x01,
	.rmii = 1,
};

static struct platform_device nec_candy_device = {
	.name = "nec_candy",
	.id = 0,
	.dev.platform_data = &nec_candy_pdata,
	.num_resources = 2,
	.resource = (struct resource[]) {
		{
			.name	= "nec_candy_regs",
			.start  = CPHYSADDR(EMMA_ETHER_BASE),
			.end    = CPHYSADDR(EMMA_ETHER_BASE + 0x400),
			.flags  = IORESOURCE_MEM,
		},
		{
			.name	= "nec_candy_irq",
			.start  = EMMA_IRQ_ETHER,
			.end    = EMMA_IRQ_ETHER,
			.flags  = IORESOURCE_IRQ,
		},

	},
};

#define EMMA_OHCI_BASE	(0x10201000)
static u64 ohci_dmamask = ~(u32)0;
static struct platform_device emma_ohci = {
	.name = "emma-ohci",
	.id = 0,
	.dev = {
		.dma_mask		= &ohci_dmamask,
		.coherent_dma_mask	= 0xffffffff,
	},
	.num_resources = 2,
	.resource = (struct resource[]) {
		{
			.name	= "ohci_mem",
			.start  = KSEG1ADDR(EMMA_OHCI_BASE),
			.end    = KSEG1ADDR(EMMA_OHCI_BASE + 0xff),
			.flags  = IORESOURCE_MEM,
		},
		{
			.name	= "ohci_irq",
			.start  = EMMA_IRQ_USB,
			.end    = EMMA_IRQ_USB,
			.flags  = IORESOURCE_IRQ,
		},

	},
};

#define EMMA_EHCI_BASE	(0x10200f00)
static u64 ehci_dmamask = ~(u32)0;
static struct platform_device emma_ehci = {
	.name = "emma-ehci",
	.id = 0,
	.dev = {
		.dma_mask		= &ehci_dmamask,
		.coherent_dma_mask	= 0xffffffff,
	},
	.num_resources = 2,
	.resource = (struct resource[]) {
		{
			.name	= "ehci_mem",
			.start  = KSEG1ADDR(EMMA_EHCI_BASE),
			.end    = KSEG1ADDR(EMMA_EHCI_BASE + 0xff),
			.flags  = IORESOURCE_MEM,
		},
		{
			.name	= "ehci_irq",
			.start  = EMMA_IRQ_USB,
			.end    = EMMA_IRQ_USB,
			.flags  = IORESOURCE_IRQ,
		},
	},
};

static struct platform_device *devices[] = {
	&i2c_emma_devices[0],
	&i2c_emma_devices[1],
	&i2c_emma_devices[2],
	&emma_ohci,
	&emma_ehci,
	&serial_emma,
	&nec_candy_device,
};

#ifdef CONFIG_MTD
static struct mtd_partition e21_parts[] = {
#if 1 // Mod for EMMA2TL Pixela Board ES1 HASE@09061802 NOR Flash上にMTDパーティション作成
	[0] = {
		.name = "OSBOURNE : Kernel(vmlinux)",
		.offset =0,
		.size = KERNEL_SIZE_N
	},
	[1] = {
		.name = "OSBOURNE : FileSystem(SQUASHFS)",
		.offset = MTDPART_OFS_APPEND,
		.size = ROOTFS_SIZE_N
	},
	[2] = {
		.name = "OSBOURNE : App_binary(SQUASHFS)",
		.offset = MTDPART_OFS_APPEND,
		.size = APP_BIN_SIZE_N
	},
	[3] = {
		.name = "OSBOURNE : Boot loader",
		.offset = MTDPART_OFS_APPEND,
		.size = B_LOADER_SIZE_N
	},
	[4] = {
		.name = "OSBOURNE : Boot Logo image",
		.offset = MTDPART_OFS_APPEND,
		.size = LOGOIMG_SIZE_N
	},
	[5] = {
		.name = "OSBOURNE : 4KEC_FW",
		.offset =MTDPART_OFS_APPEND,
		.size = FW_SIZE_N
	},
	[6] = {
		.name = "OSBOURNE : Boot setting",
		.offset =MTDPART_OFS_APPEND,
		.size = B_SETTING_SIZE_N
	},
	[7] = {
		.name = "OSBOURNE : MAC address",
		.offset =MTDPART_OFS_APPEND,
		.size = MAC_ADDR_SIZE_N
	},
#else
	[0] = {
		.name = "Free Area1",
		.offset = 0x00000000,
		.size = 0x02000000,
	},
	[1] = {
		.name = "Root FS",
		.offset = MTDPART_OFS_APPEND,
		.size = 0x01500000,
	},
	[2] = {
		.name = "kernel image",
		.offset = MTDPART_OFS_APPEND,
		.size = 0x00700000,
	},
	[3] = {
		.name = "boot code area",
		.offset = MTDPART_OFS_APPEND,
		.size = 0x00200000,
	},
	[4] = {
		.name = "boot env area",
		.offset = MTDPART_OFS_APPEND,
		.size = 0x00040000,
		.mask_flags = MTD_WRITEABLE,  /* force read-only */
	},
	[5] = {
		.name = "Free Area2",
		.offset = MTDPART_OFS_APPEND,
		.size = 0x001C0000,
	},
#endif
};
#endif

static int __init platform_devices_setup(void)
{
#ifdef CONFIG_MTD
	physmap_set_partitions(e21_parts, ARRAY_SIZE(e21_parts));
#endif
	memcpy(nec_candy_pdata.mac_addr, (void *) EMMA3TL_MAC_FLASH, 6);
	return platform_add_devices(devices, ARRAY_SIZE(devices));
}

arch_initcall(platform_devices_setup);
