/*
 *  arch/mips/emma3tl/e21/setup.c
 *      This file is setup for EMMA3TL E21.
 *
 * Author: NEC Electronics Corporation
 *
 * Copyright (C) 2008 NEC Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  This file is based on the arch/mips/ddb5xxx/ddb5477/setup.c.
 *
 *	Copyright 2001 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/console.h>
#include <linux/sched.h>
#include <linux/pci.h>
#include <linux/fs.h>		/* for ROOT_DEV */
#include <linux/ioport.h>
#include <linux/param.h>	/* for HZ */
#include <linux/major.h>
#include <linux/kdev_t.h>
#include <linux/root_dev.h>
#include <linux/serial.h>
#include <linux/serial_core.h>
#include <linux/kgdb.h>

#include <linux/cpu.h>
#include <asm/bootinfo.h>
#include <asm/addrspace.h>
#include <asm/time.h>
#include <asm/bcache.h>
#include <linux/irq.h>
#include <asm/reboot.h>
#include <asm/gdb-stub.h>
#include <asm/traps.h>
#include <asm/debug.h>

#include <asm/emma3tl/emma3tl.h>

#define	USE_CPU_COUNTER_TIMER	/* whether we use cpu counter */

extern void e21_led(const char *);
static int bus_frequency;

static void e21_machine_restart(char *command)
{
	static void (*back_to_prom) (void) = (void (*)(void))0xbfc00000;

	printk(KERN_INFO "EMMA3TL E21 restart.\n");
	e21_led("restart.");
	back_to_prom();
}

static void e21_machine_halt(void)
{
	printk(KERN_INFO "EMMA3TL E21 halted.\n");
	e21_led("halted.");
	while (1)
		;
}

static void e21_machine_power_off(void)
{
	printk(KERN_INFO
		"EMMA3TL E21 halted. Please turn off the power.\n");
	e21_led("poweroff.");
	while (1)
		;
}

  static unsigned long clock[] =
			{ 327680000, 262144000 };

static unsigned int __init detect_bus_frequency(unsigned long rtc_base)
{
	u32 reg;

	/* detect from boot strap */
	reg = emma_in32(EMMA_BHIF_STRAP_0);
	reg = (reg >> 4) & 0x1; /* EMMA3TL2用へ変更 */

	return clock[reg];
}

static void __init emma_time_init(void)
{
	if (bus_frequency == 0)
		bus_frequency = detect_bus_frequency(0);

	mips_hpt_frequency = bus_frequency / 2;
}

extern int setup_irq(unsigned int irq, struct irqaction *irqaction);

void __init plat_timer_setup(struct irqaction *irq)
{
	/* we are using the cpu counter for timer interrupts */
	setup_irq(CPU_IRQ_BASE + 7, irq);
}

static void e21_board_init(void);

#if defined(CONFIG_BLK_DEV_INITRD)
extern unsigned long __rd_start, __rd_end, initrd_start, initrd_end;
#endif

static inline void __init e21_sio_setup(void)
{
#ifdef CONFIG_KGDB_8250
	struct uart_port emma_port;

	memset(&emma_port, 0, sizeof(emma_port));

	emma_port.flags =
		UPF_BOOT_AUTOCONF | UPF_SKIP_TEST;
	emma_port.iotype = UPIO_DWAPB;
	emma_port.regshift = 2;	/* I/O addresses are every 8 bytes */
	emma_port.uartclk = 24000000;	/* Clock rate of the chip */

	emma_port.line = 0;

#if 1  /* PRD-LH132B基板では,  ttyS0(Subm通信用)=COM1  ttyS1(デバッグ用)=COM0。 暫定的に、ここで設定。*/
	emma_port.mapbase = KSEG1ADDR(EMMA_PFUR1_BASE + 3);
#else
	emma_port.mapbase = KSEG1ADDR(EMMA_PFUR0_BASE + 3);
#endif
	emma_port.membase = (u8 *)emma_port.mapbase;
	emma_port.private_data = (void *)(emma_port.membase + 0x7c);
	early_serial_setup(&emma_port);

#if 1  /* PRD-LH132B基板では,  ttyS0(Subm通信用)=COM1  ttyS1(デバッグ用)=COM0。 暫定的に、ここで設定。*/
	emma_port.irq = EMMA_IRQ_PFUR1;
#else
	emma_port.irq = EMMA_IRQ_PFUR0;
#endif

#ifdef CONFIG_KGDB_TTYS0
	kgdb8250_add_port(0, &emma_port);
#endif

	emma_port.line = 1;
#if 1  /* PRD-LH132B基板では,  ttyS0(Subm通信用)=COM1  ttyS1(デバッグ用)=COM0。 暫定的に、ここで設定。*/
	emma_port.mapbase = KSEG1ADDR(EMMA_PFUR0_BASE + 3);
#else
	emma_port.mapbase = KSEG1ADDR(EMMA_PFUR1_BASE + 3);
#endif
	emma_port.membase = (u8 *)emma_port.mapbase;
	emma_port.private_data = (void *)(emma_port.membase + 0x7c);
	early_serial_setup(&emma_port);

#if 1  /* PRD-LH132B基板では,  ttyS0(Subm通信用)=COM1  ttyS1(デバッグ用)=COM0。 暫定的に、ここで設定。*/
	emma_port.irq = EMMA_IRQ_PFUR0;
#else
	emma_port.irq = EMMA_IRQ_PFUR1;
#endif

#ifdef CONFIG_KGDB_TTYS1
	kgdb8250_add_port(1, &emma_port);
#endif

	emma_port.line = 2;
	emma_port.mapbase = KSEG1ADDR(EMMA_PFUR2_BASE + 3);
	emma_port.membase = (u8 *)emma_port.mapbase;
	emma_port.private_data = (void *)(emma_port.membase + 0x7c);
	early_serial_setup(&emma_port);

	emma_port.irq = EMMA_IRQ_PFUR2;

#ifdef CONFIG_KGDB_TTYS2
	kgdb8250_add_port(2, &emma_port);
#endif

#endif
}

void __init plat_mem_setup(void)
{
	/* initialize board - we don't trust the loader */
	e21_board_init();

	set_io_port_base(KSEG1);

	board_time_init = emma_time_init;

	_machine_restart = e21_machine_restart;
	_machine_halt = e21_machine_halt;
	pm_power_off = e21_machine_power_off;

	/* setup resource limits */
	ioport_resource.end = ~0;

	/* Reboot on panic */
	panic_timeout = 180;

#if defined(CONFIG_BLK_DEV_INITRD)
	ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0);
	initrd_start = (unsigned long)&__rd_start;
	initrd_end = (unsigned long)&__rd_end;
#endif
	e21_sio_setup();

}

static void __init e21_board_init(void)
{
	e21_led("MVL E3TL");
}
