/*
 *
 * i2c-emma3tl.h: NEC EMMA3TL global defines
 *
 * Author: NEC Electronics Corporation
 *
 * Copyright (C) 2007-2008 NEC Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef I2C_EMMA_H
#define I2C_EMMA_H

/**
 * PBRD Register Map
 **/
#define I2C_SDA_DELAY		(0x00000004)
#define I2C_FAIL_SAFE		(0x00000008)


/**
 * IIC registers map
 **/

/*---------------------------------------------------------------------------*/
/* CNT - I2C Control register (000h R/W)                                     */
/*---------------------------------------------------------------------------*/
#define MASTER_MODE		(0x00000001)
#define STANDARD_MODE		(0x00000002)
#define FAST_MODE		(0x00000004)
#define IC_MAX_SPEED_MODE	(FAST_MODE)
#define IC_SPEED		(0x00000006)
#define IC_10BITADDR_SLAVE	(0x00000008)
#define IC_10BITADDR_MASTER_RD	(0x00000010)
#define IC_RESTART_EN		(0x00000020)
#define IC_SLAVE_DISABLE	(0x00000040)

/*---------------------------------------------------------------------------*/
/* IC_TAR - I2C Target Address (004h R/W)                                    */
/*---------------------------------------------------------------------------*/
#define IC_TAR_ADDR		(0x000003FF)
#define GC_OR_START		(0x00000400)
#define IC_SPECIAL		(0x00000800)
#define IC_10BITADDR_MASTER	(0x00001000)

/*---------------------------------------------------------------------------*/
/* IC_SAR - I2C Slave Address (008h R/W)                                     */
/*---------------------------------------------------------------------------*/
#define IC_SAR_ADDR		(0x000003FF)

/*---------------------------------------------------------------------------*/
/* IC_DATA_CMD - I2C RX/TX Data Buffer and Command (010h R/W)                */
/*---------------------------------------------------------------------------*/
#define IC_DAT			(0x000000FF)
#define IC_CMD			(0x00000100)

/*---------------------------------------------------------------------------*/
/* IC_SS_SCL_HCNT - Standard Speed I2C Clock SCL High Count (014h R/W)       */
/*---------------------------------------------------------------------------*/
#define SS_SCL_HCNT		(0x0000FFFF)

/*---------------------------------------------------------------------------*/
/* IC_SS_SCL_LCNT - Standard Speed I2C Clock SCL LOW Count (018h R/W)        */
/*---------------------------------------------------------------------------*/
#define SS_SCL_LCNT		(0x0000FFFF)

/*---------------------------------------------------------------------------*/
/* IC_FS_SCL_HCNT - Fast Speed I2C Clock SCL High Count (01Ch R/W)           */
/*---------------------------------------------------------------------------*/
#define FS_SCL_HCNT		(0x0000FFFF)

/*---------------------------------------------------------------------------*/
/* IC_FS_SCL_LCNT - Fast Speed I2C Clock Low High Count (020h R/W)           */
/*---------------------------------------------------------------------------*/
#define FS_SCL_LCNT		(0x0000FFFF)

/*---------------------------------------------------------------------------*/
/* IC_INTR_STAT - I2C Interrupt Status (002Ch R)                             */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/* IC_INTR_MASK - I2C Interrupt Mask (030h R/W)                              */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/* IC_RAW_INTR_STAT - I2C Raw Interrupt Stasus (034h R/W)                    */
/*---------------------------------------------------------------------------*/
#define INTR_RX_UNDER		(0x00000001)
#define INTR_RX_OVER		(0x00000002)
#define INTR_RX_FULL		(0x00000004)
#define INTR_TX_OVER		(0x00000008)
#define INTR_TX_EMPTY		(0x00000010)
#define INTR_RD_REQ		(0x00000020)
#define INTR_TX_ABRT		(0x00000040)
#define INTR_RX_DONE		(0x00000080)
#define INTR_ACTIVITY		(0x00000100)
#define INTR_STOP_DET		(0x00000200)
#define INTR_START_DET		(0x00000400)
#define INTR_GEN_CALL		(0x00000800)
#define INTR_STAT_MASK		(0x00000FFF)

/*---------------------------------------------------------------------------*/
/* IC_RX_TL - I2C Receive FIFO Threshold (038h R/W)                          */
/*---------------------------------------------------------------------------*/
#define RX_TL			(0x000000FF)

/*---------------------------------------------------------------------------*/
/* IC_TX_TL - I2C Transmit FIFO Threshold (03Ch R/W)                         */
/*---------------------------------------------------------------------------*/
#define TX_TL			(0x000000FF)

/*---------------------------------------------------------------------------*/
/* IC_CLR_INTR - Clear Combined and Individual Interrupts (040h R)           */
/*---------------------------------------------------------------------------*/
#define CLR_INTR		(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_CLR_RX_UNDER - Clear RX_UNDER Interrupt (044h R)                       */
/*---------------------------------------------------------------------------*/
#define CLR_RX_UNDER		(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_CLR_RX_OVER - Clear RX_OVER Interrupt (048h R)                         */
/*---------------------------------------------------------------------------*/
#define CLR_RX_OVER		(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_CLR_TX_OVER - Clear TX_OVER Interrupt (04Ch R)                         */
/*---------------------------------------------------------------------------*/
#define CLR_TX_OVER		(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_CLR_RD_REQ - Clear RD_REQ Interrupt (050h R)                           */
/*---------------------------------------------------------------------------*/
#define CLR_RD_REQ		(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_CLR_TX_ABRT - Clear TX_ABRT Interrupt (054h R)                         */
/*---------------------------------------------------------------------------*/
#define CLR_TX_ABR		(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_CLR_RX_DONE - Clear RX_DONE Interrupt (058h R)                         */
/*---------------------------------------------------------------------------*/
#define CLR_RX_DONE		(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_CLR_ACTIVITY - Clear ACTIVITY Interrupt (05Ch R)                       */
/*---------------------------------------------------------------------------*/
#define CLR_ACTIVITY		(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_CLR_STOP_DET - Clear STOP_DET Interrupt (060h R)                       */
/*---------------------------------------------------------------------------*/
#define CLR_STOP_DET		(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_CLR_START_DET - Clear START_DET Interrupt (064h R)                     */
/*---------------------------------------------------------------------------*/
#define CLR_START_DET		(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_CLR_GEN_CALL - Clear GEN_CALL Interrupt (068h R)                       */
/*---------------------------------------------------------------------------*/
#define CLR_GEN_CALL		(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_ENABLE - I2C Enable (06Ch R/W)                                         */
/*---------------------------------------------------------------------------*/
#define ENABLE			(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_STATUS - I2C Status Register (070h R)                                  */
/*---------------------------------------------------------------------------*/
#define ACTIVITY_STATUS		(0x00000001)
#define TFNF			(0x00000002)
#define TFE			(0x00000004)
#define RFNE			(0x00000008)
#define RFF			(0x00000010)
#define MST_ACTIVITY		(0x00000020)
#define SLV_ACTIVITY		(0x00000040)

/*---------------------------------------------------------------------------*/
/* IC_TXFLR - Transmit FIFO Level Register (074h R)                          */
/*---------------------------------------------------------------------------*/
#define TXFLR			(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_RXFLR - Receive FIFO Level Register (078h R)                           */
/*---------------------------------------------------------------------------*/
#define RXFLR			(0x00000001)

/*---------------------------------------------------------------------------*/
/* IC_TX_ABRT_SOURCE - I2C Transmit Abort Status Register (080h R/W)         */
/*---------------------------------------------------------------------------*/
#define ABRT_7B_ADDR_NOACK	(0x00000001)
#define ABRT_10ADDR1_NOACK	(0x00000002)
#define ABRT_10ADDR2_NOACK	(0x00000004)
#define ABRT_TXDATA_NOACK	(0x00000008)
#define ABRT_GCALL_NOACK	(0x00000010)
#define ABRT_GCALL_READ		(0x00000020)
#define ABRT_HS_ACKDET		(0x00000040)
#define ABRT_SBYTE_ACKDET	(0x00000080)
#define ABRT_HS_NORSTRT		(0x00000100)
#define ABRT_SBYTE_NORSTRT	(0x00000200)
#define ABRT_10B_RD_NORSTRT	(0x00000400)
#define ABR_MASTER_DIS		(0x00000800)
#define ABR_LOST		(0x00001000)
#define ABRT_SLVFLUSH_TXFIFO	(0x00002000)
#define ABRT_SLV_ARBLOST	(0x00004000)
#define ABRT_SLVRD_INTX		(0x00008000)

/*---------------------------------------------------------------------------*/
/* IC_ENABLE_STATUS - I2C Enable Status Register (09Ch R/W)                  */
/*---------------------------------------------------------------------------*/
#define IC_EN				(0x00000001)
#define SLV_RX_ABORTED			(0x00000002)
#define SLV_FIFO_FILLED_AND_FLUSHED	(0x00000004)

/***********************************************************************
 * I2C registers
 ***********************************************************************
 */
#define IC_CON (0x000)/* I2C Control */
#define IC_TAR			(0x004) /* I2C Target Address */
#define IC_SAR			(0x008) /* I2C Slave Address */
#define IC_DATA_CMD		(0x010) /* I2C RX/TX Data Buffer and Command */

/* Standard Speed I2C Clock SCL High Count */
#define IC_SS_SCL_HCNT		(0x014)

/* Standard Speed I2C Clock SCL Low Count */
#define IC_SS_SCL_LCNT		(0x018)

/* Fast Speed I2C Clock SCL High Count */
#define IC_FS_SCL_HCNT		(0x01C)

#define IC_FS_SCL_LCNT		(0x020) /* Fast Speed I2C Clock SCL Low Count */
#define IC_INTR_STAT		(0x02C) /* I2C Interrupt Status */
#define IC_INTR_MASK		(0x030) /* I2C Interrupt Mask */
#define IC_RAW_INTR_STAT	(0x034) /* I2C Raw Interrupt Status */
#define IC_RX_TL		(0x038) /* I2C Receive FIFO Threshold */
#define IC_TX_TL		(0x03C) /* I2C Transmit FIFO Threshold */

/* Clear Combined and Individual Interrupts */
#define IC_CLR_INT		(0x040)

#define IC_CLR_RX_UNDER		(0x044) /* Clear RX_UNDER Interrupt */
#define IC_CLR_RX_OVER		(0x048) /* Clear RX_OVER Interrupt */
#define IC_CLR_TX_OVER		(0x04C) /* Clear TX_OVER Interrupt */
#define IC_CLR_RD_REQ		(0x050) /* Clear RD_REQ Interrupt */
#define IC_CLR_TX_ABRT		(0x054) /* Clear TX_ABRT Interrupt */
#define IC_CLR_RX_DONE		(0x058) /* Clear RX_DONE Interrupt */
#define IC_CLR_ACTIVITY		(0x05C) /* Clear ACTIVITY Interrupt */
#define IC_CLR_STOP_DET		(0x060) /* Clear STOP_DET Interrupt */
#define IC_CLR_START_DET	(0x064) /* Clear START_DET Interrupt */
#define IC_CLR_GEN_CALL		(0x068) /* Clear GEN_CALL Interrupt */
#define IC_ENABLE		(0x06C) /* I2C Enable */
#define IC_STATUS		(0x070) /* I2C Status Register */
#define IC_TXFLR		(0x074) /* Transmit FIFO Level Register */
#define IC_RXFLR		(0x078) /* Receive FIFO Level Register */
#define IC_TX_ABRT_SOURCE	(0x080) /* I2C Transmit Abort Status Register */
#define IC_ENABLE_STATUS	(0x09C) /* I2C Enable Status Register */
#endif /* I2C_EMMA_H */
