/*
 * emma2tl_sd.h
 * SD card controller driver for emma2tl
 *
 * M.Ikeda
 * 2009.07.08
 *
 * Copyright 2009 PIXELA CORPORATION. All rights reserved.
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/mm.h>
#include <linux/interrupt.h>
#include <linux/dma-mapping.h>

#include <linux/mmc/host.h>
#include <linux/mmc/protocol.h>
#include <asm/io.h>
#include <asm/emma3tl/emma3tl.h>

#include "emma2tl_sd.h"

#define DRIVER_NAME "emma-sd"

#define	DEBUG

#ifdef	DEBUG
#define	DBG(fmt,args...)	printk(KERN_INFO DRIVER_NAME ": " fmt,##args)
#else
#define	DBG(fmt,args...)	
#endif

/**
 * @brief	挿入検出信号の割り込みハンドラ
 */
static irqreturn_t emmasd_irq(int irq, void *dev_id, struct pt_regs *regs)
{
	DBG("irq called\n");

	/* 割り込みを一旦無効にする */
	disable_irq(EMMA_IRQ_INT90);

	if (emma_in32(EMMA_GPIO_DATA) & EMMA_GPIO_BIT_INSERT) {
		printk(KERN_INFO DRIVER_NAME ": card removed!\n");
	} else {
		printk(KERN_INFO DRIVER_NAME ": card insert!\n");
	}

	/* 割り込み要因を解除 */
	emma_clr32(EMMA_GPIO_INT_STAT, ~EMMA_GPIO_BIT_INSERT);	/* 要因クリア */

	/* 割り込みを有効とする */
	enable_irq(EMMA_IRQ_INT90);

	return	0;
}

static int __devinit emmasd_probe(struct platform_device *pdev)
{
	int		ret;

	DBG("start probing..\n");

	/*
	 * 挿入検出信号の割り込みハンドラ登録
	 */
	ret = request_irq(EMMA_IRQ_INT90, emmasd_irq, 0, "SDC", 0);
	if (ret < 0) {
		printk(KERN_ERR DRIVER_NAME "couldn't request irq=%d, ret=%d\n",
			EMMA_IRQ_INT90, ret);
		return	-ENXIO;
	}

	/* 他の初期化が終わるまで、割り込みを一旦無効にする */
	disable_irq(EMMA_IRQ_INT90);

	/*
	 * 挿入検出信号のGPIO割り込みを設定する
	 */
	emma_clr32(EMMA_GPIO_INT_MODE, ~EMMA_GPIO_BIT_INSERT);	/* エッジトリガ */
	emma_set32(EMMA_GPIO_INT_COND_A, EMMA_GPIO_BIT_INSERT);
	emma_set32(EMMA_GPIO_INT_COND_B, EMMA_GPIO_BIT_INSERT);	/* 両エッジ検出 */
	emma_set32(EMMA_GPIO_INT_MASK, EMMA_GPIO_BIT_INSERT);	/* マスク解除 */

	/* 割り込みを有効とする */
	enable_irq(EMMA_IRQ_INT90);

	return	0;
}

static int __devexit emmasd_remove(struct platform_device *pdev)
{
	return	0;
}

static struct platform_device *emmasd_device;
static struct platform_driver emmasd_driver = {
	.probe		= emmasd_probe,
	.remove		= emmasd_remove,
	.suspend	= NULL,
	.resume		= NULL,
	.driver		= {
		.name	= DRIVER_NAME,
	},
};

static int __init emmasd_init(void)
{
	int	result;

	printk(KERN_INFO DRIVER_NAME "driver init\n");

	result = platform_driver_register(&emmasd_driver);
	if (result < 0) {
		return	result;
	}

	emmasd_device = platform_device_alloc(DRIVER_NAME, -1);
	if (!emmasd_device) {
		platform_driver_unregister(&emmasd_driver);
		return	-ENOMEM;
	}

	result = platform_device_add(emmasd_device);
	if (result) {
		platform_device_put(emmasd_device);
		platform_driver_unregister(&emmasd_driver);
		return	result;
	}
	return	0;
}

static void __init emmasd_exit(void)
{
	printk(KERN_INFO DRIVER_NAME "driver exit\n");

	platform_driver_unregister(&emmasd_driver);
}

module_init(emmasd_init);
module_exit(emmasd_exit);

#ifdef MODULE
MODULE_AUTHOR("Pixela corporation");
MODULE_DESCRIPTION("SD driver for the Emma2TL");
MODULE_LICENSE("GPL");
#endif
