/*
 *	emma2tl_sd.h
 *	SD card controller driver for emma2tl
 *
 *	M.Ikeda
 *	2009.07.08
 *
 *	Copyright 2007 PIXELA CORPORATION. All rights reserved.
 */

#ifndef	__EMMA2TL_SD_H
#define	__EMMA2TL_SD_H

/******************************************************************************
 SDコントローラ H/W定義
******************************************************************************/

/* レジスタベース */
#define	EMMA_SD_BASE_ADDR		0x10111200

/*
 * レジスタオフセット
 */
#define	EMMA_SD_CONTROL0		(EMMA_SD_BASE_ADDR + 0x00)
#define	EMMA_SD_CONTROL1		(EMMA_SD_BASE_ADDR + 0x04)
#define	EMMA_SD_STATUS			(EMMA_SD_BASE_ADDR + 0x08)
#define	EMMA_SD_CMD_RES0		(EMMA_SD_BASE_ADDR + 0x0c)
#define	EMMA_SD_CMD_RES1		(EMMA_SD_BASE_ADDR + 0x10)
#define	EMMA_SD_CMD_RES2		(EMMA_SD_BASE_ADDR + 0x14)
#define	EMMA_SD_DATA			(EMMA_SD_BASE_ADDR + 0x18)
#define	EMMA_SD_CRC_STATUS		(EMMA_SD_BASE_ADDR + 0x1c)
#define	EMMA_SD_CRC_DAT0		(EMMA_SD_BASE_ADDR + 0x20)
#define	EMMA_SD_CRC_DAT1		(EMMA_SD_BASE_ADDR + 0x24)
#define	EMMA_SD_CRC_DAT2		(EMMA_SD_BASE_ADDR + 0x28)
#define	EMMA_SD_CRC_DAT3		(EMMA_SD_BASE_ADDR + 0x2c)
#define	EMMA_SD_INT_CONTROL		(EMMA_SD_BASE_ADDR + 0x34)
#define	EMMA_SD_INT_STATUS		(EMMA_SD_BASE_ADDR + 0x38)
#define	EMMA_SD_DMA_CONTROL		(EMMA_SD_BASE_ADDR + 0x40)
#define	EMMA_SD_DMA_STATUS		(EMMA_SD_BASE_ADDR + 0x44)

/*
 * コントロールレジスタ0
 */
#define	EMMA_SD_CONTROL0_MSK		0x00000001
#define	EMMA_SD_CONTROL0_CRCSEL		0x00000020
#define	EMMA_SD_CONTROL0_CLKE		0x00000040
#define	EMMA_SD_CONTROL0_DLINE		0x00000080

/*
 * コントロールレジスタ1
 */
#define	EMMA_SD_CONTROL1_RST		0x00000040
#define	EMMA_SD_CONTROL1_DMAE		0x00010000

/* SD I/F 動作モード */
#define	EMMA_SD_CARDMODE_CLKOUT		0	/* クロック出力のみ */
#define	EMMA_SD_CARDMODE_INHIBIT	1	/* 設定禁止 */
#define	EMMA_SD_CARDMODE_CMD_WR512	2	/* コマンド + データライト512bit */
#define	EMMA_SD_CARDMODE_CMD_RD512	3	/* コマンド + データリード512bit */
#define	EMMA_SD_CARDMODE_DAT_WR512	4	/* データライト512bit */
#define	EMMA_SD_CARDMODE_DAT_RD512	5	/* データリード512bit */
#define	EMMA_SD_CARDMODE_CMD		6	/* コマンド(レスポンス無し */
#define	EMMA_SD_CARDMODE_CMD_RES48	7	/* コマンド(レスポンス48bit) */
#define	EMMA_SD_CARDMODE_CMD_RES136	8	/* コマンド(レスポンス136bit) */
#define	EMMA_SD_CARDMODE_CMD_DAT_RD512	9	/* コマンド + データリード(512bit) */
#define	EMMA_SD_CARDMODE_CMD_DAT_RD64	10	/* コマンド + データリード(64bit) */
#define	EMMA_SD_CARDMODE_CMD_DAT_RD32	11	/* コマンド + データリード(32bit) */

/* SD I/F 動作オプション */
#define	EMMA_SD_CARDOF_SHIFT_BIT	4

#define EMMA_SD_CARDOP_8CLK		0	
#define EMMA_SD_CARDOP_32CLK		1	
#define EMMA_SD_CARDOP_128CLK		2	
#define EMMA_SD_CARDOP_256CLK		3	
#define	EMMA_SD_CARDOP_CRC_OUT		0	
#define	EMMA_SD_CARDOP_CRC_AUTOMAKE	1	

/*
 * ステータスレジスタ
 */
#define	EMMA_SD_STATUS_BSY		0x00000000	/* 処理中 */
#define	EMMA_SD_STATUS_RBEMP		0x00000001	/* 受信バッファ空 */
#define	EMMA_SD_STATUS_RBFULL		0x00000002	/* 受信バッファフル */
#define	EMMA_SD_STATUS_TBEMP		0x00000004	/* 送信バッファ空 */
#define	EMMA_SD_STATUS_TBFULL		0x00000010	/* 送信バッファフル */
#define	EMMA_SD_STATUS_DAT0		0x00000020	/* SDカード入力信号 */
#define	EMMA_SD_STATUS_CDONE		0x00000040	/* コマンド転送完了 */
#define	EMMA_SD_STATUS_RDONE		0x00000080	/* レスポンス受信完了 */

/*
 * 割り込みコントロール／ステータスレジスタ
 */
#define	EMMA_SD_INT_BSYINTEN		0x00000010	/* 動作完了割り込み許可 */
#define	EMMA_SD_INT_DBERRINTEN		0x00010000	/* エラー発生割り込み許可 */

/******************************************************************************
 GPIOコントローラ H/W定義
******************************************************************************/

#define	EMMA_GPIO_BIT_INSERT		0x00000002	/* 挿入検出信号 LowActive */



#endif	/* __EMMA2TL_SD_H */
